#!/usr/bin/env python

# Rss Notify
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Authors: Isaiah Heyer (freshapplepy@gmail.com)

# Copyright (C) 2009 Isaiah Heyer

import gtk
import gobject

import os
import sys
import threading
import time

import rss_notify

builder = gtk.Builder()
glade_file = os.path.join(rss_notify.GLADE_PATH, "gui.glade")
try:
    builder.add_from_file(glade_file)

except Exception, data:
    print data
    print "error loading glade file"
    sys.exit(1)

KEY = 0
ELEMENTS = {}

def add_element(element):
    global KEY
    global ELEMENTS

    KEY += 1
    ELEMENTS[KEY] = element
    return KEY

get = builder.get_object

class Window(object):

    def __init__(self):
        
        self.window = get("window")

        self.import_last_dir = None
        self.export_last_dir = None


        # treeview stuff
        treeview = get("treeview")
        self.treeview = treeview
        self.selection = treeview.get_selection()
        self.selection.set_mode(gtk.SELECTION_MULTIPLE)

        treestore = get("treestore")
        treestore.set_sort_func(10, self.sort_treestore)
        treestore.set_sort_column_id(10, gtk.SORT_ASCENDING)

        self.treestore = treestore

        self.FM = rss_notify.get_feedmanager()

        feed_icon = os.path.join(rss_notify.IMAGE_PATH,
                "standard.svg")

        self.feed_icon_pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(
                feed_icon, 16, 16)

        self.dir_pixbuf = self.window.render_icon(gtk.STOCK_DIRECTORY,
                        gtk.ICON_SIZE_MENU)

        self.populate_treestore()
        
                

        ## connect signals
        builder.connect_signals(self)


    def sort_treestore(self, model, iter1, iter2, data=None):
        v31 = model.get_value(iter1, 3)
        v32 = model.get_value(iter2, 3)

        if v31 > v32:
            return -1
        elif v31 < v32:
            return 1


        v11 = model.get_value(iter1, 1)
        v12 = model.get_value(iter2, 1)
        if v11:
            v11 = v11.upper()
        if v12:
            v12 = v12.upper()

        if v11 < v12:
            return -1
        elif v11 > v12:
            return 1

        return 0

    def populate_treestore(self):

        ELEMENTS = {}
        KEY = 0

        self.categories = []

         ## re-expand rows
        expanded_rows = []
        def for_expanded(treeview, path, data=None):
            expanded_rows.append(path)

        self.treeview.map_expanded_rows(for_expanded, None)



        treestore = self.treestore
        treestore.clear()



        def trav(element, piter=None):
            
            if element.get('type') == "rss":

                pixbuf = self.feed_icon_pixbuf

                key = add_element(element)
                piter = treestore.append(piter, [pixbuf,
                    element.get("title"), key, 0])


            elif element.get('type') == "folder":

                pixbuf = self.dir_pixbuf

                key = add_element(element)

                piter = treestore.append(piter, [pixbuf,
                    element.get("title"), key, 1])
        


                def get_category(iter):
                    pi = treestore.iter_parent(iter)
                    n = treestore.get_value(iter, 1)
                    if pi:
                        n = get_category(pi)+" > "+n

                    return n

                v = [pixbuf, get_category(piter), key]

                self.categories.append(v)
        
            

            for child in element:
                trav(child, piter)


        root_outline = self.FM.get_feed_outlines()
        if root_outline != None:
            for child in root_outline:
                trav(child)

        for path in expanded_rows:
            self.treeview.expand_row(path, False)

    # Callbacks

    def on_window_delete_event(self, window, event, data=None):

        self.window.hide_on_delete()
        return True

    def on_treeviewcolumn_clicked(self, tvc, data=None):
        sort_order = tvc.get_sort_order()
        if sort_order == gtk.SORT_ASCENDING:
            sort_order = tvc.set_sort_order(gtk.SORT_DESCENDING)
            self.treestore.set_sort_column_id(10, gtk.SORT_DESCENDING)
        else:
            sort_order = tvc.set_sort_order(gtk.SORT_ASCENDING)
            self.treestore.set_sort_column_id(10, gtk.SORT_ASCENDING)

    def on_AddFeed_activate(self, action, data=None):
        # update categories
        self.populate_treestore()

        dialog = AddFeedDialog(self.categories)
        dialog.dialog.set_transient_for(self.window)

        ret_val = dialog.run()
        dialog.dialog.destroy()
            
        if ret_val:
            text = ret_val[0]
            category = ret_val[1]

            element = self.FM.add_feed(text, category)
            if element != False:
                add_element(element)
                self.populate_treestore()
                self.FM.save_feeds()
            else:
                dialog = gtk.MessageDialog(parent=self.window,
                    flags=gtk.DIALOG_DESTROY_WITH_PARENT,
                    type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_CLOSE,
                    message_format="Error: Invalid Feed URL")



                response = dialog.run()
                dialog.destroy()

    def on_RemoveFeed_activate(self, action, data=None):
       
        model, paths = self.selection.get_selected_rows()
        paths.reverse()
        for path in paths:
            iter = model.get_iter(path)
            element = ELEMENTS[model.get_value(iter, 2)]
            self.FM.remove_feed(element)
            model.remove(iter)

        self.FM.save_feeds()


    
    def filechooser(self, title, action):
        if action == gtk.FILE_CHOOSER_ACTION_OPEN:
            buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                    gtk.STOCK_OPEN, gtk.RESPONSE_OK)
        else:
            buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                    gtk.STOCK_SAVE, gtk.RESPONSE_OK)
        
        dialog = gtk.FileChooserDialog(title, self.window,
                action, buttons)

        return dialog

    def on_import_menu_item_activate(self, menuitem, data=None):
        dialog = self.filechooser("Import Feed List",
                gtk.FILE_CHOOSER_ACTION_OPEN)

        dialog.set_transient_for(self.window)
        dialog.set_do_overwrite_confirmation(True)

        filter = gtk.FileFilter()
        filter.set_name("All files")
        filter.add_pattern("*")
        dialog.add_filter(filter)

        filter = gtk.FileFilter()
        filter.set_name("OPML Files")
        filter.add_mime_type("text/x-opml+xml")
        filter.add_pattern("*.opml")

        dialog.add_filter(filter)
        dialog.set_filter(filter)

        if self.import_last_dir:
            dialog.set_current_folder(self.import_last_dir)
        else:
            dialog.set_current_folder(os.getenv("HOME"))

        response = dialog.run()

        if response == gtk.RESPONSE_OK:
            filename = dialog.get_filename()
            try:
                self.FM.import_feeds(filename)
                self.populate_treestore()
            except Exception, data:
                err_dialog = gtk.MessageDialog(parent=self.window,
                    flags=gtk.DIALOG_DESTROY_WITH_PARENT,
                    type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_CLOSE,
                    message_format="Error: Invalid Feed File")

                response = err_dialog.run()
                self.import_last_dir = dialog.get_current_folder()
                dialog.destroy()
                err_dialog.destroy()
                return

        self.import_last_dir = dialog.get_current_folder()

        dialog.destroy()

    def on_export_menu_item_activate(self, menuitem, data=None):

        dialog = self.filechooser("Export Feed List",
                gtk.FILE_CHOOSER_ACTION_SAVE)
        dialog.set_transient_for(self.window)

        if not self.export_last_dir:
            dialog.set_current_folder(os.getenv("HOME"))

        dialog.set_current_name("feedlist.opml")

        response = dialog.run()
        
        if response == gtk.RESPONSE_OK:
            filename = dialog.get_filename()
            try:
                self.FM.export_feeds(filename)
            except Exception, data:
                print data

        self.export_last_dir = dialog.get_current_folder()

        dialog.destroy()


    def on_Preferences_activate(self, action, data=None):
        prefs = Preferences()
        prefs.window.set_transient_for(self.window)
        prefs.show_all()

    def on_About_activate(self, action, data=None):
        dialog = AboutDialog()
        dialog.dialog.set_transient_for(self.window)
        dialog.run()
        dialog.dialog.destroy()

    def on_Quit_activate(self, action, data=None):
        gtk.main_quit()
        sys.exit(0)
    
    def on_close_button_clicked(self, button, data=None):
        self.window.hide()
    
    def toggle_visible(self):

        if self.window.get_property("visible"):
            self.window.hide()
        else:
            self.window.show_all()
   
class AddFeedDialog(object):

    def __init__(self, categories):

        builder = gtk.Builder()
        glade_file = os.path.join(rss_notify.GLADE_PATH, "misc.glade")
        try:
            builder.add_from_file(glade_file)

        except Exception, data:
            print data
            print "error loading glade file"
            sys.exit(1)

        get = builder.get_object

        self.dialog = get("add_dialog")
        self.entry = get("add_dialog_entry")
        self.cbb = get("add_dialog_combobox")

        self.cat_dialog = get("cat_dialog")
        self.cat_dialog_entry = get("cat_dialog_entry")
        self.cat_dialog_cbb = get("cat_dialog_combobox")


        self.cat_liststore = get("cat_liststore")
        
        for c in categories:
            self.cat_liststore.append(c)

        self.FM = rss_notify.get_feedmanager()

        ## add a "new" category

        self.dir_pixbuf = self.dialog.render_icon(gtk.STOCK_DIRECTORY,
                        gtk.ICON_SIZE_MENU)
        self.new_pixbuf = self.dialog.render_icon(gtk.STOCK_NEW,
                        gtk.ICON_SIZE_MENU)

        self.cat_liststore.prepend([None, "None", 0])
        self.cat_liststore.append([self.new_pixbuf, "New...", 0])
        
        self.cbb.set_active(0)
        self.cat_dialog_cbb.set_active(0)

        builder.connect_signals(self)

    def on_add_dialog_combobox_changed(self, combobox, data=None):

        model = combobox.get_model()

        iter = combobox.get_active_iter()
        sel_text = None
        if iter:
            sel_text = model.get_value(
                    combobox.get_active_iter(), 1)
        
        if sel_text == "New...":
            
            for row in model:
                if row[1] == "New...":
                    model.remove(row.iter)
                    break


            response = self.cat_dialog.run()
            self.cat_dialog.hide()
            
            name = self.cat_dialog_entry.get_text()
            name = name.strip()
            if not name:
                return

            parent_key = self.cat_liststore.get_value(
                    self.cat_dialog_cbb.get_active_iter(), 2)

            if parent_key:
                parent = ELEMENTS[parent_key]
            else:
                parent = None

            element = self.FM.add_category(name, parent)
            
            key = add_element(element)

            model.append([self.dir_pixbuf, name, key])
            model.append([self.new_pixbuf, "New...", 0])

            self.cbb.set_active(len(model) - 2)

    def run(self):
        
        response = self.dialog.run()

        text = ""

        if response == gtk.RESPONSE_OK:
            text = self.entry.get_text()
            text = text.strip()
        
            self.entry.set_text("")
        
            key = self.cat_liststore.get_value(self.cbb.get_active_iter(),
                    2)

            category = None
            if key != 0:
                category = ELEMENTS[key]

            return (text, category)
            

class AboutDialog(object):
    
    def __init__(self):

        builder = gtk.Builder()
        glade_file = os.path.join(rss_notify.GLADE_PATH, "misc.glade")
        try:
            builder.add_from_file(glade_file)

        except Exception, data:
            print data
            print "error loading glade file"
            sys.exit(1)

        get = builder.get_object

        self.dialog = get("about_dialog")

    def run(self):
        response = self.dialog.run()
        return response
        
                

class Preferences(object):

    def __init__(self):

        ## update gui from configuration
        self.conf = rss_notify.get_configurator()
        builder = gtk.Builder()

        glade_file = os.path.join(rss_notify.GLADE_PATH, "prefs.glade")
        try:
            builder.add_from_file(glade_file)

        except Exception, data:
            print data
            print "error loading glade file"
            sys.exit(1)

        get = builder.get_object

        self.window = get("window")

        rb1 = get("rb1")
        rb2 = get("rb2")

        i = self.conf.get("notification_icon").get_int()
        if i == 0:
            rb1.set_active(True)
        elif i == 1:
            rb2.set_active(True)

        auto_cb = get("autostart_cb")
        auto_cb.set_active(self.conf.get("autostart").get_bool())

        adj1 = get("adjustment1")
        adj1.set_value(self.conf.get("update_interval").get_int())

        adj2 = get("adjustment2")
        adj2.set_value(self.conf.get("num_of_items").get_int())

        builder.connect_signals(self)

    def on_rb1_toggled(self, rb, data=None):
        if rb.get_active():
            self.conf.set("notification_icon", 0)
    
    def on_rb2_toggled(self, rb, data=None):
        if rb.get_active():
            self.conf.set("notification_icon", 1)
    

    def on_autostart_cb_toggled(self, cb, data=None):
        self.conf.set("autostart", cb.get_active())
        if cb.get_active():
            rss_notify.add_to_startup()
        else:
            rss_notify.remove_from_startup()

    def on_adjustment1_value_changed(self, adj, data=None):
        self.conf.set("update_interval", int(adj.get_value()))

    def on_adjustment2_value_changed(self, adj, data=None):
        self.conf.set("num_of_items", int(adj.get_value()))

    def on_close_button_clicked(self, button, data=None):

        self.window.destroy()

    def show_all(self):
        self.window.show_all()
     
_menu = get("status_icon_menu")
def get_status_icon_menu():
    return _menu
